#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/site/update_db.py $
$Id: update_db.py 29620 2007-03-15 20:19:23Z dbinger $

When we update the live site, this is our procedure.
Make a tarball of the running code and database file.
This is the emergency kit.
Copy the revised code into place.

If the code revisions require corresponding data revisions, 
  Copy the database file to a temporary name.
  Run an update_db.py script that changes attribute names, classes,
  or whatever is needed to convert the database to agree with the 
  new version of the code.  
  qp -d <site>
  Move the old database file away.  Move the fixed version into place.
  qp -u <site>
Otherwise,
  there are no data changes, so we just 
  qp -q <site>

This script is intended to serve as an example of an update_db.py
for a site named "dulcinea".
"""
from qp.lib.site import Site
import sys
connection = root = commit = pack = trace = None
# This call sets connection, root, commit, pack, and trace.
Site('dulcinea').update_db_environment(globals(), *sys.argv[1:])

@trace
def repickle():
    # We have changed some classes from old-style to new-style.
    # By itself, that is not a problem, but we have also made the
    # new-style classes use slots for attributes, and that means
    # that a __setstate__ is required to unpickle the old-style
    # pickles.  With this function, we re-pickle all objects that
    # contain references to these classes, and then the __setstate__
    # methods will not be needed.
    # First hack in a temporary __setststate__ method.
    def __setstate__(self, state):
        for key, value in state.items():
            setattr(self, key, value)
    from dulcinea.physical_value import PhysicalValue
    from dulcinea.range_value import RangeValue
    from dulcinea.tolerance import Tolerance
    from dulcinea.history import Event
    from dulcinea.survey import Answer
    PhysicalValue.__setstate__ = __setstate__
    RangeValue.__setstate__ = __setstate__
    Tolerance.__setstate__ = __setstate__
    Event.__setstate__ = __setstate__
    Answer.__setstate__ = __setstate__
    # Now repickle everything that appears to contain an instance of one of
    # these classes.
    from durus.connection import touch_every_reference
    touch_every_reference(
        connection, 'PhysicalValue', 'RangeValue', 'Tolerance',
        'Event', 'Answer')


# Uncomment this if you want the script to run repickle().
#repickle()

commit() # probably required.
pack() # optional, but nice if you have time for it.  



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_links.py $
$Id: utest_links.py 29256 2007-01-30 18:21:19Z dbinger $
"""
from dulcinea.contact import Contact
from dulcinea.links import LinkItem, LinkFolder
from dulcinea.links import sort_by_title, sort_reverse_by_date
from sancho.utest import UTest


class LinksTest (UTest):

    class AdminContact(Contact):

        def is_admin(self):
            return True

    def check_link(self):
        self.link = LinkItem()
        self.link.set_link_url('http://www.example.org')
        self.link.get_link_url()
        self.link.set_text('hi')
        assert self.link.get_text() == 'hi'
        self.link.set_email('hi')
        assert self.link.get_email() == 'hi'
        self.link.as_text()
        self.link.as_email()
        assert self.link.is_approved() == False
        self.link.set_approved(True)
        assert self.link.is_approved() == True
        self.link.get_allowed_mime_types()
        self.link.get_allowed_mime_types(user=self.AdminContact('superuser'))
        assert self.link.get_local_url() == '/links/None/'
        self.link2 = LinkItem()
        [self.link, self.link2].sort(sort_reverse_by_date)
        [self.link, self.link2].sort(sort_by_title)

    def check_link_folder(self):
        self.link_db = LinkFolder()
        self.link_db.get_category('foo')
        self.link_db.links_with_category('foo')


if __name__ == "__main__":
    LinksTest()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_stored_file.py $
$Id: utest_stored_file.py 27756 2005-12-16 20:07:59Z dbinger $
"""
import os, shutil
from datetime import datetime
from sancho.utest import UTest, raises
from dulcinea.stored_file import path_check, new_file
from dulcinea.user import DulcineaUser

TEST_ROOT_DIR = '/tmp/storedfiletest'
TEST_CONTENT = "Contents of a file go here."

import dulcinea.stored_file
def get_file_store():
    return TEST_ROOT_DIR
dulcinea.stored_file.get_file_store = get_file_store


assert dulcinea.stored_file.get_file_store() == TEST_ROOT_DIR

class StoredFileTest (UTest):

    def _pre(self):
        if os.path.exists(TEST_ROOT_DIR):
            shutil.rmtree(TEST_ROOT_DIR)
        os.mkdir(TEST_ROOT_DIR)
        self.user = DulcineaUser('a')
        self.content_filename = os.path.join(TEST_ROOT_DIR, 'test_content')
        content = open(self.content_filename, 'w')
        content.write(TEST_CONTENT)
        content.close()

    def _post(self):
        shutil.rmtree(TEST_ROOT_DIR)
        del self.content_filename

    def check_file(self):
        sf = new_file(open(self.content_filename))
        self.f = sf.open()
        assert self.f.read() == TEST_CONTENT
        self.f.close()
        assert sf.get_size() == len(TEST_CONTENT)
        raises(ValueError, sf.set_mime_type, "textplain")
        sf.set_mime_type("text/plain")
        sf.set_filename("foo")
        assert sf.get_filename() == 'foo'
        sf.set_description("boo")
        assert sf.get_description() == 'boo'
        u = DulcineaUser(None)
        sf.set_owner(u)
        assert sf.get_owner() == u
        assert sf.has_manage_access(u)
        adate = datetime.now()
        sf.set_date(adate)
        assert sf.get_date() == adate


    def check_path_check(self):
        assert path_check("Abc") == 'abc'
        assert path_check("abc/../../../etc/passwd") == None
        assert path_check("/special/characters/-_@.cif") == (
                      "/special/characters/-_@.cif")
        assert path_check("/illegal/characters\001/") == None


if __name__ == "__main__":
    StoredFileTest()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/util.py $
$Id: util.py 29786 2007-04-25 15:24:15Z rmasse $
"""
from datetime import datetime
from distutils.fancy_getopt import wrap_text
from formatter import AbstractFormatter, DumbWriter
from htmllib import HTMLParser
from os.path import isdir, dirname, abspath
from qpy import h8, stringify
from StringIO import StringIO
import re
import time


SAFE_TAGS = ['<p>', '</p>', '<b>', '</b>', '<i>', '</i>', '<ul>',
             '</ul>', '<li>', '</li>', '<br>', '<pre>', '</pre>',
             '<strong>', '</strong>']

def _htmlescape_except(text, safe_tags):
    """(text) -> h8

    Quotes 'text' for use in an HTML page, except for the tags
    listed in 'safe_tags' which are considered safe (e.g. <p>, <b>).
    Both upper and lower case versions of the tags will be applied
    """
    if text is None:
        return None
    text = stringify(h8.quote(text))
    for tag in safe_tags:
        tag_re = re.compile(stringify(h8.quote(tag)), re.IGNORECASE)
        text = tag_re.sub(tag, text)
    return h8(text)

# re to automatically hyperlink email addresses and URLs
_link_re = re.compile(r"""\b(
    # email address
      [\w.-]+ # local part
      @
      [\w.-]+\.[\w]{2,4} # domain
    |
    # URL
      (?:https?://|www\.) # must start with http or www
      [\w.-]+\.[\w]{2,4} # domain
      (?::\d+)? # optional port
      (?:/[\w#$%&+,-./:;=?@\[\]^_|~]*)? # optional path
   )""", re.VERBOSE)

def activate_links(text):
    """(text:str|None) -> h8|None
    Returns an h8.quoted version of text, with things that look like email
    addresses and URLs turned into hyperlinks.
    """
    def _link_replace(m):
        text = url = m.group(0)
        extra = ""
        if text.find("@") == -1:
            if text[-1] in ".,":
                # don't include as part of the URL (easier to handle here
                # than in the regex)
                extra = text[-1]
                text = text[:-1]
            if not text.startswith("http"):
                url = "http://" + text
        else:
            if not text.startswith("mailto"):
                url = "mailto:" + text
        return '<a href="%s">%s</a>%s' % (url, text, extra)

    if text is None:
        return None
    return h8(_link_re.sub(_link_replace, stringify(h8.quote(text))))


def format_text(text, safe_tags=SAFE_TAGS):
    """(text : string, safe_tags : [str]) -> h8

    Convert an ASCII string containing text into HTML.
    The resulting HTML has been reformatted in the following ways:
       * <, &, and > are escaped
       * Things that look like email addresses and URLs will be turned into
         hyperlinks.
       * If safe_tags is defined, it is expected to be a list of tags
         considered to be safe (e.g. <p>, <b>, etc).  These tags will
         not be escaped.
    """
    return activate_links(_htmlescape_except(text, safe_tags))

def sanitize_url(url):
    """(url:string) -> string
    Try to ensure a URL is well-formed, by adding http:// if it isn't present.
    """
    if url is None:
        return None
    if '@' in url and not url.startswith('mailto:'):
        # assume it's an e-mail address
        url = "mailto:" + url
    elif url.find(":") == -1:
        # assume http:// is missing
        url = "http://" + url
    return url

_paragraph_re = re.compile('\n\n+')

def split_paragraphs(text):
    return _paragraph_re.split(text)

def wrap_paragraphs(text):
    """(text) -> string
    Wrap a sequence of paragraphs for output as plain text.
    """
    if text is None:
        return ""
    return '\n\n'.join([wrap_paragraph(paragraph)
                        for paragraph in split_paragraphs(text)])

def wrap_paragraph(text):
    line_length = 70
    if isinstance(text, unicode):
        lines = [text.__class__(x, 'utf-8')
                 for x in wrap_text(text.encode('utf-8'), line_length)]
    else:
        lines = wrap_text(text, line_length)
    return '\n'.join(lines)

def insert_paragraph_tags(text):
    """(text:string|None) -> h8|None

    Prepare a text field for display as HTML.
    Currently this just HTML quotes the string and then inserts
    <p> tags at blank lines.  
    """
    if text is None:
        return None
    return h8(
        '<p>' +
        _paragraph_re.sub('</p><p>', stringify(h8.quote(text))) +
        '</p>')

def datetime_to_int(date_time):
    """(date_time:datetime) -> int
    Returns the number of seconds since the epoch.
    """
    return int(time.mktime(date_time.timetuple()))

def beginning_of_next_month(date_time):
    """(date_time:datetime) -> datetime
    Return a datetime for the exact beginning of the month
    following the given date_time.  
    """
    year = date_time.year
    month = date_time.month + 1
    if month == 13:
        year += 1
        month = 1
    return datetime(year=year, month=month, day=1, hour=0, minute=0,
                    second=0, microsecond=0)

def html2txt(text):
    """(any) -> str
    """
    sio = StringIO()
    parser = HTMLParser(AbstractFormatter(DumbWriter(sio)))
    parser.feed(stringify(text).encode('utf8'))
    result = sio.getvalue()
    return result.decode('latin1')

def is_new(persistent_object):
    """(persistent_object : durus.persistent.Persistent) -> boolean
    """
    return persistent_object._p_connection is None

def get_module_directory(module):
    result = abspath(module.__file__)
    if isdir(result):
        return result
    else:
        return dirname(result)



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/script_util.py $
$Id: script_util.py 27812 2006-01-13 16:52:28Z dbinger $
"""
from optparse import OptionParser
import sys, os, time, pdb


class Debugger(pdb.Pdb):
    """
    Calling the constructor of this class puts you in the
    interactive debugger.  This is like pdb.post_mortem, except
    that the traceback is printed and the exit command works.
    """
    def __init__(self):
        pdb.Pdb.__init__(self)
        (exc_type, exc_value, tb) = sys.exc_info()
        up = 0
        if tb is None:
            try:
                raise
            except:
                (exc_type, exc_value, tb) = sys.exc_info()
            down = 1
        self.reset()
        while tb.tb_next is not None:
            tb = tb.tb_next
        self.interaction(tb.tb_frame, tb)

    def interaction(self, frame, traceback):
        self.setup(frame, traceback)
        self.curindex = self.curindex - 1
        self.curframe = self.stack[self.curindex][0]
        self.lineno = None
        self.do_explain()
        self.cmdloop()
        self.forget()

    def do_explain(self, arg=None):
        print "\n"
        for stack_entry in self.stack[self.curindex-3:-1]:
            self.print_stack_entry(stack_entry)
        print
        self.do_args(None)
        print
        self.do_list("%s,13" % (self.curframe.f_lineno - 12))

    def help_explain(self):
        print """
Print a slice of the stack, args to this function, and list the list the
section of code being executed.
"""

    def do_exit(self, arg):
        os._exit(1)

    def help_exit(self):
        print """Terminate this process."""


def _verbose(arg, _depth=1):
    """Print arg, eval(arg), and then report the time it took."""
    frame = sys._getframe(_depth)
    start = time.time()
    print arg
    result = eval(arg, frame.f_globals, frame.f_locals)
    finish = time.time()
    print '%s completed in %1.3f seconds.' % (arg, finish-start)
    return result

def _verbose_catch(arg):
    """
    Like verbose(), but drop into the debugger
    on exceptions.
    """
    try:
        _verbose(arg, _depth=2)
    except SystemExit:
        raise
    except:
        print arg, 'FAILED'
        Debugger()

def verbose_main(fun):
    """
    verbose_main parses a command line and calls fun
    with a first argument set to either verbose
    or else to verbose_catch.
    Command line arguments not declared as options here are passed on
    as arguments to fun.
    """
    usage = "usage: %prog [options] [dbfile]"
    parser = OptionParser(usage)
    parser.add_option("-c", "--catch-errors", action="store_true",
                      help=("catch exceptions from update code and "
                            "drop into pdb (default: let it crash"))
    (options, args) = parser.parse_args()

    if options.catch_errors:
        fun(_verbose_catch, *args)
    else:
        fun(_verbose, *args)

def no_rerun(root, release_id):
    if root.get('version') == release_id:
        print "This update has already been executed."
        raise SystemExit
    else:
        root['version'] = release_id



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/cents.py $
$Id: cents.py 29619 2007-03-15 18:44:42Z dbinger $
"""
from qp.lib.spec import either, require, specify, Specified

class Cents (Specified):

    cents_is = either(int, long)

    def __init__(self, value=0):
        specify(self, cents=value)

    def get_cents(self):
        return self.cents

    def __str__(self):
        return "$%.02f" % self.get_dollars()

    def get_dollars(self):
        return self.cents / 100.0

    def __add__(self, other):
        require(other, Cents)
        return Cents(self.cents + other.cents)

    def __sub__(self, other):
        require(other, Cents)
        return Cents(self.cents - other.cents)

    def __cmp__(self, other):
        require(other, Cents)
        return cmp(self.cents, other.cents)




"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/property/property_list.py $
$Id: property_list.py 29610 2007-03-14 21:12:59Z dbinger $
"""
from dulcinea.base import DulcineaPersistent
from dulcinea.property.property import Property
from dulcinea.property.property_template import PropertyTemplate
from qp.lib.spec import either, require


class PropertyList (DulcineaPersistent):

    list_is = [either(Property, PropertyTemplate)]

    def __init__(self, values=None):
        self.list = []
        if values:
            for value in values:
                self.add(value)

    def __copy__(self):
        result = self.__class__()
        result.list = self.list[:]
        return result

    def __eq__(self, other):
        require(other, PropertyList)
        if len(self) != len(other):
            return False
        def as_dict(property_list):
            return dict((p.get_name(), p) for p in property_list)
        return as_dict(self) == as_dict(other)

    def __str__(self):
        return "PropertyList(%s)" % self.list

    def __len__(self):
        return len(self.list)

    def __getitem__(self, index):
        return self.list[index]

    def __add__(self, other):
        newlist = PropertyList()
        if isinstance(other, PropertyList):
            newlist.list = self.list + other.list
        else:
            newlist.list = self.list[:]
            for item in other:
                newlist.add(item)
        return newlist

    def get_names(self):
        """() -> [string]

        Return the list of all names in the list.
        """
        return [property.get_name() for property in self]

    def find(self, name):
        """(name : string) -> any

        Searches the list for a value with name 'name'.  Returns the
        matching value, or None if it couldn't find one.
        """
        for property in self.list:
            if name == property.get_name():
                return property
        require(name, basestring)
        return None

    def add(self, property):
        """(property : Property|PropertyTemplate)

        Raises ValueError if this list already has a value with the same name.
        """
        require(property, either(Property, PropertyTemplate))
        if self.find(property.get_name()) is not None:
            raise ValueError, \
                  "list already has a value named %r" % property.get_name()
        self._p_note_change()
        self.list.append(property)

    def remove(self, name):
        """(name : string)

        Find the value named by 'name' and remove it.
        """
        require(name, basestring)
        for index, property in enumerate(self):
            if name == property.get_name():
                self._p_note_change()
                del self.list[index]

    def __iter__(self):
        for item in self.list:
            yield item

    def _get_instances(self, klass):
        return [p for p in self.list if isinstance(p, klass)]

    def get_value_properties(self):
        """Return a new PropertyList containing all properties from the
        current list that have values.
        """
        result = PropertyList()
        result.list = self._get_instances(Property)
        return result

    def get_templates(self):
        """Return a new PropertyList containing all properties from the
        current list that don't have values.
        """
        result = PropertyList()
        result.list = self._get_instances(PropertyTemplate)
        return result

    def get_visible(self):
        result = PropertyList()
        result.list = [property for property in self
                       if not property.is_hidden()]
        return result
